#! /usr/bin/perl -w

use strict;
use Getopt::Std;
use Data::Dumper;

BEGIN {
require lib;
  # Try to guess the location of the HHS library directory:
  lib->import("$ENV{HHSInstall}/lib") if(defined($ENV{HHSInstall}));
  $0 =~ /\A(.*\/)/;
  if(defined($1)) {
    lib->import("$1/../lib");
  } else {
    die "Cannot get basedir out of \"$0\"\n";
  }

  eval "
    require hwrf_tracks;
    hwrf_tracks->import();
  ";
  if($@) {
    die "Unable to load the hwrf_tracks module: $@.  Check your \$HHSInstall environment variable and try again.  Aborting";
  }
};

if($#ARGV != 5) {
  die "Syntax:
  $0 /path/to/b/deck basin1 BB NN YYYY MMDDHH

Where:
  /path/to/b/deck -- path to b deck directory with bBBYYYY.dat files
  basin1 -- one-letter tcvitals basin for output purposes only
  BB NN YYYY MMDDHH -- search parameters: basin, storm number, year,
    month day and hour.  Use / as a wildcard.  Perl regular expressions
    are allowed.\n"
}

my ($bdeckdir,$basin1,$srbasin,$srnum,$sryear,$srmdh)=@ARGV;

$basin1=uc($basin1);

our (@contents,$basin,$num,$cyc,$year);
my %prevname;
my %prevdep;
sub tcvitals_action {
    my $line;
    warn "GOT HERE: $line";
    sub t($$) { # t() clips or pads general text
      my($s,$l)=@_;
      die unless defined($l);
      my $r=0;
      if($l<0) {
        $r=1;
        $l=-$l;
      }
      if(!defined($s) || -z $s) {
        return '0' x $l;
      }
      die unless defined($s);
      if(length($s)>$l) {
        if($r==0) { # cut from left
          return substr($s,length($s)-$l,$l);
        } else { # cut from right
          return substr($s,0,$l);
        }
      } elsif(length($s)<$l) {
        if($r==0) { # pad at right
          return($s.(' 'x($l-length($s))));
        } else { # pad at left
          return(('0'x($l-length($s))).$s);
        }
      } else {
        return $s;
      }
    }
    sub g($$) { # g() clips or pads lats & lons (g=geolocations)
      my($s,$l)=@_;
      die unless defined($l);
      if($l<0) {
        $l=-$l;
      }
      if(!defined($s) || -z $s) {
        return '0' x $l;
      }
      die unless defined($s);
      if(length($s)>$l) {
        return substr($s,0,$l);
      } elsif(length($s)<$l) {
        return(('0'x($l-length($s))).$s);
      } else {
        return $s;
      }
    }
    sub n($$) { # n() clips or pads numbers
      my ($s,$l)=@_;
      die unless defined($l);
      if(!defined($s) || -z $s || !$s=~/\A(-?)(\d+)\z/) {
        return '-'.('9' x ($l-1));
      }
warn 'S UNDEFINED' unless defined($s);
    warn 'L UNDEFINED' unless defined($l);
      if(length($s)>$l) {
        die "Cannot fit $s in a $l-length region.\n";
      } elsif( length($s)==$l) {
        return $s;
      } elsif(defined($1) && $1 ne '') {
        return( ('0'x($l-length($s))."$s"));
      } else {
        return(('0'x($l-length($s))).$s);
      }
    }
    sub v($$) { # v() clips or pads velocities and converts kn->m/s
      my $kn2mps=0.51444444444444;
      if(defined($_[0]) && $_[0]=~/\A\d+\z/) {
        return n(int(0.5+$_[0]*$kn2mps),$_[1]);
      } else {
        return n('',$_[1]);
      }
    }
    sub r($$) { # r() clips or pads radii and converts nmi->km
      my $nm2km=1.852;
      if(defined($_[0]) && $_[0]=~/\A\d+\z/) {
        return n(int(0.5+$_[0]*$nm2km),$_[1]);
      } else {
        return n('',$_[1]);
      }
    }
    my ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
       $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
       $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
       $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4);
    my (%radii,%ordered,$center);
    foreach $line (@hwrf_tracks::contents) {
#      my $line2=$line;
#      chomp $line2;
#      print "> $line2\n";
      my @f=split(/\s*,\s*/,$line);
      ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
       $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
       $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
       $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4)=@f;
      my $center='NHC ';
      if(defined($rad) && $rad ne "") {
          if($rad>=33 && $rad<=36) {
              $radii{$ymdh}[1]=[@f];
          } elsif($rad>=48 && $rad<=52) {
              $radii{$ymdh}[2]=[@f];
          } elsif($rad>=63 && $rad<=69) {
              $radii{$ymdh}[3]=[@f];
          }
      }
      push @{$ordered{$ymdh}},[@f];
    }
    my $key;
    my $prevname=$prevname{$hwrf_tracks::basin}{$hwrf_tracks::num}{$hwrf_tracks::year};
    my $prevdep=$prevdep{$hwrf_tracks::basin}{$hwrf_tracks::num}{$hwrf_tracks::year};
    if(!defined($prevname)) {
      $prevname='UNKNOWN';
    }
    my ($savemrd,$saveradp,$saverrp)=(undef,undef,undef);
    my ($savemslp,$savevmax)=(undef,undef);
    foreach $key(sort { $a <=> $b } keys %ordered) {
      my @all;
      my @ord=@{$ordered{$key}};
      if(defined($radii{$key})) {
        @all=@{$radii{$key}};
      }
      my $savestormname='UNKNOWN';
      my $savedep='X';
      foreach $a (@{$ordered{$key}}) {
        ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
         $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
         $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
         $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4)=@{$a};

        $saveradp=$radp if(defined($radp) && $radp ne "" && $radp+0>0);
        $savemrd=$mrd if(defined($mrd) && $mrd ne "" && $mrd+0>0);
        $saverrp=$rrp if(defined($rrp) && $rrp ne "" && $rrp+0>0);
        $savemslp=$mslp if(defined($mslp) && $mslp ne "" && $mslp+0>0);
        $savevmax=$vmax if(defined($vmax) && $vmax ne "" && $vmax+0>0);

        if(!defined($stormname) || -z $stormname) {
            if(defined($initials) && ! -z $initials) {
                # Old format, so stormname is shifted back a few fields.
                $stormname=$initials;
            }
        }
        if(defined($depth) && $depth=~/\A[DMS]\z/ && $savedep eq 'X') {
          $savedep=$depth;
        }
        if($stormname eq 'INVEST') {
          if($savestormname eq 'UNKNOWN') {
            $savestormname=$stormname;
          }
        } else {
          $savestormname=$stormname;
        }
      }
      if($savedep eq 'X') {
        if(defined($prevdep)) {
          $savedep=$prevdep;
        } else {
          if(defined($savevmax) && $savevmax > 30) {
            $savedep='D';
          } elsif(defined($savevmax) && $savevmax<15) {
            $savedep='S';
          } elsif(defined($savevmax)) {
            $savedep='M';
          } else { # vmax unknown, depth unknown
            $savedep='D';
          }
        }
      } else {
        $prevdep{$hwrf_tracks::basin}{$hwrf_tracks::num}{$hwrf_tracks::year}=$savedep;
      }
      if($savestormname eq 'UNKNOWN') {
        $savestormname=$prevname;
      } else {
        $prevname{$hwrf_tracks::basin}{$hwrf_tracks::num}{$hwrf_tracks::year}=$savestormname;
      }
      $savestormname='INVEST' if($savestormname eq 'UNKNOWN');

      ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
       $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
       $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
       $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4)=@{$ord[$#ord]};
      my $hh=n(substr($ymdh,8,2),2);
      my $out;
      next unless($hh eq '00' || $hh eq '06' || $hh eq '12' || $hh eq '18');
      if($qbasin eq 'AL' || $qbasin eq 'EP' || $qbasin eq 'CP') {
        $center='NHC';
      }
      ($mrd,$radp,$rrp,$mslp,$vmax)=($savemrd,$saveradp,$saverrp,$savemslp,$savevmax);
      
      if(!defined($radp) || -z $radp || $radp<$mslp+1) {
        $radp=$mslp+2;
      }

      if($radp<995) {
        $radp=995;
      }
      $out=join(' ',(t($center,4),n($cy,2).t($basin1,1),
                t($savestormname,9),n(substr($ymdh,0,8),8),n(substr($ymdh,8,2),2) . '00',
                g($lat,4),g($lon,5),n($dir,3),n($speed,3),n($mslp,4),n($radp,4),
                r($rrp,4),v($vmax,2),r($mrd,3))).' ';
      if(ref($all[1])) {
        ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
            $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
            $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
            $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4)=@{$all[1]};
        $out.=join(' ',r($rad1,4),r($rad2,4),r($rad3,4),r($rad4,4));
      } else {
        $out.='-999 -999 -999 -999';
      }
      my $a;
      $out.=" $savedep ";
      if(ref($all[2])) {
         ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
            $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
            $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
            $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4)=@{$all[2]};
        $out.=join(' ',r($rad1,4),r($rad2,4),r($rad3,4),r($rad4,4));
      } else {
        $out.='-999 -999 -999 -999';
      }
      $out.=' -9 -99N -999W ';
      if(ref($all[3])) {
 ($qbasin,$cy,$ymdh,$technum,$tech,$tau,$lat,$lon,$vmax,$mslp,
            $ty,$rad,$windcode,$rad1,$rad2,$rad3,$rad4,$radp,$rrp,$mrd,
            $gusts,$eye,$subregion,$maxseas,$initials,$dir,$speed,
            $stormname,$depth,$seas,$seascode,$seas1,$seas2,$seas3,$seas4)=@{$all[3]};
        $out.=join(' ',r($rad1,4),r($rad2,4),r($rad3,4),r($rad4,4));
      } else {
        $out.='-999 -999 -999 -999';
      }
      print "$out\n";
    }
    return;
}

#set_verbosity(1==1);
our $query_matches;
warn "HERE: $srbasin - $srnum - $sryear - $srmdh - $bdeckdir\n";
my $ret=search_a_deck( 0,$bdeckdir,\&tcvitals_action,$srbasin,$srnum,$sryear,$srmdh,'CARQ','a' );

